import {Component, OnInit} from "@angular/core";
import {properties} from "../../../../environments/environment";
import {ActivatedRoute, Router} from "@angular/router";
import {Meta, Title} from "@angular/platform-browser";
import {SEOService} from "../../sharedComponents/SEO/SEO.service";

@Component({
  selector: 'indicator-themes-page',
  template: `
		<div class="uk-section">
			<div class="uk-container">
				<h1>Indicator Themes<span class="uk-text-primary">.</span></h1>
			</div>
			<div class="uk-section uk-container">
				<div class="uk-grid uk-grid-large uk-child-width-1-2@m uk-child-width-1-1 uk-flex-middle" uk-grid>
					<div>
					  <img src="assets/common-assets/monitor-assets/indicator-themes-circle.png">
					</div>
					<div>
						<div class="uk-width-4-5@m">
							<h5>Indicator themes that we are covering in the Monitor dashboards.</h5>
							<p>
								Please be aware that this is the current set, which will be enriched as requests and data are coming into the
                <a href="https://graph.openaire.eu" class="text-graph" target="_blank">OpenAIRE Research Graph</a>. We are in your disposal to add more indicator themes that may fit your needs.
							</p>
							<p>
								Check out the indicator pages (for <a [routerLink]="['../funder']" [relativeTo]="route">funders</a>,
                <a [routerLink]="['../organization']" [relativeTo]="route">research institutions</a> and
                <a [routerLink]="['../ri']" [relativeTo]="route">research initiatives</a>)
                for the specific indicators for each type of dashboard, and the  <a [routerLink]="['../../methodology']" [relativeTo]="route">methodology and terminology</a> page on how we produce the metrics.
							</p>
						</div>
					</div>
				</div>
			</div>
		</div>
  `
})
export class IndicatorThemesComponent implements OnInit {
  public properties = properties;
  
  constructor(private router: Router,
              private meta: Meta,
              private title: Title,
              private seoService: SEOService,
              public route: ActivatedRoute) {
  }
  
  ngOnInit() {
    const description = "Monitor | Indicator Themes";
		const title = "Monitor | Indicator Themes";
		this.metaTags(title, description);
  }
  
  metaTags(title, description) {
    const url = properties.domain + properties.baseLink + this.router.url;
    this.seoService.createLinkForCanonicalURL(url, false);
    this.meta.updateTag({content: url}, "property='og:url'");
    this.meta.updateTag({content: description}, "name='description'");
    this.meta.updateTag({content: description}, "property='og:description'");
    this.meta.updateTag({content: title}, "property='og:title'");
    this.title.setTitle(title);
  }
}
