import {Component, Input, OnChanges, OnInit, SimpleChanges} from "@angular/core";
import {ResultPreview} from "./result-preview";
import {EnvProperties} from "../properties/env-properties";
import {RouterHelper} from "../routerHelper.class";
import {AlertModal} from "../modal/alert";
import {properties} from "../../../../environments/environment";
import {Session} from "../../login/utils/helper.class";
import {Identifier, StringUtils} from "../string-utils.class";
import {OpenaireEntities} from "../properties/searchFields";

@Component({
  selector: 'result-preview',
  templateUrl: 'result-preview.component.html',
	styleUrls: ['result-preview.component.less']
})
export class ResultPreviewComponent implements OnInit, OnChanges {
  @Input() result: ResultPreview;
  @Input() properties: EnvProperties;
  public openaireEntities = OpenaireEntities;
  @Input() showSubjects: boolean = true;
  @Input() showOrganizations: boolean = true;
  @Input() modal: AlertModal = null;
  @Input() promoteWebsiteURL: boolean = false;
  @Input() hasLink: boolean = true;
	@Input() isCard: boolean = false;
  public routerHelper: RouterHelper = new RouterHelper();
  public urlParam: string;
  public url: string;
  @Input() externalUrl: string;
  public type: string;
  public beforeTitle: string[] = [];
  public dataProviderUrl = properties.searchLinkToDataProvider.split('?')[0];
  @Input() showOrcid: boolean = true;
  @Input() showEnermaps: boolean = false;
  @Input() deposit: boolean = false;

  ngOnInit(): void {
    if(this.hasLink) {
      if (this.result.resultType === "publication") {
        this.urlParam = "articleId";
        this.url = properties.searchLinkToPublication.split('?')[0];
      } else if (this.result.resultType === "dataset") {
        this.urlParam = "datasetId";
        this.url = properties.searchLinkToDataset.split('?')[0];
      } else if (this.result.resultType === "software") {
        this.urlParam = "softwareId";
        this.url = properties.searchLinkToSoftwareLanding.split('?')[0];
      } else if (this.result.resultType === "other") {
        this.urlParam = "orpId";
        this.url = properties.searchLinkToOrp.split('?')[0];
      } else if (this.result.resultType == "project") {
        if(this.result.id) {
          this.urlParam = "projectId";
        }else if(this.result.code && this.result.funderShortname){
          this.result.id = this.result.code;
          this.urlParam = "grantId";
        }
        this.url = properties.searchLinkToProject.split('?')[0];
      } else if (this.result.resultType == "organization") {
        this.urlParam = "organizationId";
        this.url = properties.searchLinkToOrganization.split('?')[0];
      } else if (this.result.resultType == "dataprovider") {
        if(this.result.compatibilityUNKNOWN && properties.adminToolsPortalType == "eosc") {
          this.urlParam = "serviceId";
          this.url = properties.searchLinkToService.split('?')[0];
        } else {
          this.urlParam = "datasourceId";
          this.url = properties.searchLinkToDataProvider.split('?')[0];
        }
      } else if (this.result.resultType == "service") {
        this.urlParam = "serviceId";
        this.url = properties.searchLinkToService.split('?')[0];
      } else {
        this.urlParam = "id";
        this.url = properties.searchLinkToResult.split('?')[0];
      }
      this.checkPID();
    }
    this.initBeforeTitle();
    if(this.result.languages) {
      this.result.languages = this.removeUnknown(this.result.languages);
    }
    if(this.result.countries) {
      this.result.countries = this.removeUnknown(this.result.countries);
    }
  }
  
  ngOnChanges(changes: SimpleChanges) {
    if(changes.result && this.hasLink) {
      this.checkPID();
    }
  }
  
  checkPID() {
    // if result has a pid use it as parameter instead of openaireId
    let pid:Identifier =this.getPID();
    if(pid){
      this.urlParam = "pid";
      this.result.id = pid.id;
    
    }
  }
  getPID() {
    return Identifier.getPIDFromIdentifiers(this.result.identifiers);
  }

  public initBeforeTitle() {
    if(this.result.resultType && this.result.resultType !== 'dataprovider') {
      this.type = this.getTypeName(this.result.resultType);
    }
    if(this.result.types) {
      this.removeUnknown(this.removeDuplicates(this.result.types)).forEach(type => {
        this.beforeTitle.push(type);
      });
    }
    if(this.result.year) {
      this.beforeTitle.push(this.result.year.toString());
    }
    if(this.result.startYear && this.result.endYear) {
      this.beforeTitle.push(this.result.startYear.toString() + ' - ' + this.result.endYear.toString());
    }
    if(this.result.provenanceAction) {
      this.beforeTitle.push(this.result.provenanceAction);
    }
    if(this.result.relationName) {
      this.beforeTitle.push(this.result.relationName);
    }
    // if(this.result.percentage) {
    //   this.beforeTitle.push((this.result.relation ? this.result.relation+": " : "") + this.result.percentage.toString() + "%");
    // }
  }
  
  public getTypeName(type: string): string {
    return StringUtils.getEntityName(type, false);
  }
  
  public removeUnknown(array: string[]): string[] {
    return array.filter(value => value.toLowerCase() !== 'unknown');
  }
  
  public removeDuplicates(array: string[]): string[] {
    return array.filter(value => value.toLowerCase() !== this.result.resultType);
  }
  
  public accessClass(accessMode: string): string {
    if(accessMode.toLowerCase().indexOf('open') !== -1) {
      return 'success';
    } else if(accessMode.toLowerCase() === 'not available') {
      return 'disabled'; // unknown
    } else {
      return 'disabled'; // closed
    }
  }
  
  public onClick() {
    if(this.modal) {
      this.modal.cancel();
    }
  }
  createParam(){
    if(this.urlParam == "grantId" ){
      return this.routerHelper.createQueryParams([this.urlParam,"funder"],[this.result.id,this.result.funderShortname])

    }
    return this.routerHelper.createQueryParam(this.urlParam,this.result.id)

  }

  public get isResultType() {
    return this.result.resultType == "publication" || this.result.resultType == "dataset" ||
      this.result.resultType == "software" || this.result.resultType == "other" || this.result.resultType == "result";
  }
}
